@@ -21,12 +21,11 @@ void net_get_peer (struct net_data* data, char* buffer, size_t bufsiz);
 void net_cleanup (struct net_data* data);
 
 #if OPENOBEX_TCPOBEX
-int tcp_setup(struct net_data*, const char*, uint16_t, const char*);
+int tcp_setup(struct net_data*, const char*, uint16_t);
 #else /* OPENOBEX_TCPOBEX */
 int inet_setup(struct net_data* data);
-#define tcp_setup(data, addr, port, intf) inet_setup(data)
 #endif /* OPENOBEX_TCPOBEX */
 
-int bluetooth_setup(struct net_data*, uint8_t);
+int bluetooth_setup(struct net_data*, char* device_addr, uint8_t);
 
 int irda_setup(struct net_data*, char*);
